<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_NameYourPrice
 * @copyright   Copyright (c) Mageplaza (https://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\NameYourPrice\Controller\Index;

use Exception;
use Magento\Catalog\Helper\Image;
use Magento\Catalog\Model\Product;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Data\Form\FormKey\Validator;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Stdlib\DateTime;
use Mageplaza\NameYourPrice\Block\Product\View\Bargain;
use Mageplaza\NameYourPrice\Helper\Data as HelperData;
use Mageplaza\NameYourPrice\Helper\Email;
use Mageplaza\NameYourPrice\Model\RequestsFactory;
use Mageplaza\NameYourPrice\Model\ResourceModel\RequestsFactory as ResourceFactory;
use Magento\Bundle\Model\Product\Type as Bundle;
use Magento\GroupedProduct\Model\Product\Type\Grouped;

/**
 * Class Request
 * @package Mageplaza\NameYourPrice\Controller\Index
 */
class Request extends Action
{
    /**
     * @var HelperData
     */
    protected $_helperData;

    /**
     * @var RequestsFactory
     */
    protected $_requestsFactory;

    /**
     * @var Bargain
     */
    protected $_bargain;

    /**
     * @var TypeListInterface
     */
    protected $_cache;

    /**
     * @var ResourceFactory
     */
    protected $_resourceFactory;

    /**
     * @var Email
     */
    protected $_email;

    /**
     * @var Image
     */
    protected $_imageHelper;

    /**
     * @var Validator
     */
    protected $formKeyValidator;

    /**
     * Request constructor.
     *
     * @param Context $context
     * @param HelperData $helperData
     * @param RequestsFactory $requestsFactory
     * @param Bargain $bargain
     * @param TypeListInterface $cache
     * @param ResourceFactory $resourceFactory
     * @param Email $email
     * @param Image $imageHelper
     * @param Validator $formKeyValidator
     */
    public function __construct(
        Context $context,
        HelperData $helperData,
        RequestsFactory $requestsFactory,
        Bargain $bargain,
        TypeListInterface $cache,
        ResourceFactory $resourceFactory,
        Email $email,
        Image $imageHelper,
        Validator $formKeyValidator
    ) {
        $this->_helperData      = $helperData;
        $this->_requestsFactory = $requestsFactory;
        $this->_bargain         = $bargain;
        $this->_cache           = $cache;
        $this->_resourceFactory = $resourceFactory;
        $this->_email           = $email;
        $this->_imageHelper     = $imageHelper;
        $this->formKeyValidator = $formKeyValidator;

        parent::__construct($context);
    }

    /**
     * @return ResponseInterface|ResultInterface
     * @throws Exception
     */
    public function execute()
    {
        $dataForm = $this->_request->getPostValue();
        $redirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        $redirect->setUrl($this->_redirect->getRefererUrl());

        if ($dataForm && $this->formKeyValidator->validate($this->getRequest())) {
            $data           = $this->getData($dataForm);
            $requestFactory = $this->_resourceFactory->create();
            $sku            = ($dataForm['product_type'] === Grouped::TYPE_CODE
                || $dataForm['product_type'] === Bundle::TYPE_CODE)
                ? array_keys($dataForm['product_id']) : $data['sku'];

            if ($requestFactory->checkUnique($sku, $dataForm['customer_email'])) {
                $this->messageManager->addErrorMessage(__('This email has bargained for this product.'));

                return $redirect;
            }

            $request = $this->_requestsFactory->create();

            try {
                if ($dataForm['product_type'] === Grouped::TYPE_CODE
                    || $dataForm['product_type'] === Bundle::TYPE_CODE) {
                    $requestFactory->addMultipleRequest($data);
                } else {
                    $request->addData($data)->save();
                }

                $this->sendMail($dataForm);
                $this->messageManager->addSuccessMessage(
                    __('This request has been sent to the admin. Please wait for the response.')
                );
            } catch (Exception $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            }
        } else {
            $this->messageManager->addErrorMessage(__('Invalid Form Key.'));
        }

        return $redirect;
    }

    /**
     * @param array $dataForm
     *
     * @return mixed
     * @throws Exception
     */
    public function getData($dataForm)
    {
        $data = [];

        if (isset($dataForm['product_type'])
            && ($dataForm['product_type'] === Grouped::TYPE_CODE || $dataForm['product_type'] === Bundle::TYPE_CODE)) {
            foreach ($dataForm['product_id'] as $key => $item) {
                if (!$dataForm['bargain_price'][$key]) {
                    continue;
                }

                $data[] = $this->processData(
                    $dataForm,
                    $key,
                    $dataForm['bargain_price'][$key],
                    $dataForm['bargain_qty'][$key]
                );
            }
        } else {
            return $this->processData(
                $dataForm,
                $dataForm['product_id'],
                $dataForm['bargain_price'],
                $dataForm['bargain_qty']
            );
        }

        return $data;
    }

    /**
     * @param array $dataForm
     * @param int $productId
     * @param float $bargainPrice
     * @param int $bargainQty
     *
     * @return array
     */
    public function processData($dataForm, $productId, $bargainPrice, $bargainQty)
    {
        $productInfo = $this->_helperData->getProductInfo($productId);

        $data['product_id']           = $productId;
        $data['product_name']         = $productInfo['name'];
        $data['sku']                  = $productInfo['sku'];
        $data['options_configurable'] = isset($dataForm['options_configurable'])
            ? $dataForm['options_configurable']
            : null;
        $data['original_price']       = $productInfo['price'];
        $data['bargain_price']        = $bargainPrice;
        $data['bargain_qty']          = $this->_helperData->isBargainQty() ? $bargainQty : 1;
        $data['status']               = HelperData::STATUS_PENDING;
        $data['customer_name']        = $dataForm['customer_name'];
        $data['customer_email']       = $dataForm['customer_email'];
        $data['store_ids']            = isset($dataForm['store_ids']) ? $dataForm['store_ids'] : '';
        $data['submitted_date']       = date(DateTime::DATETIME_PHP_FORMAT);
        $data['parent_id']            = isset($dataForm['parent_id']) ? $dataForm['parent_id'] : '';

        if (isset($dataForm['options']) && is_array($dataForm['options'])) {
            $data['options_bundle'] = HelperData::jsonEncode($dataForm['options']);
        }
        $fields = explode(',', $this->_helperData->getAdditional());
        foreach ($fields as $field) {
            if ($this->_bargain->checkAdditionalInfo($field)) {
                $data[$field] = $dataForm[$field];
            }
        }

        return $data;
    }

    /**
     * @param array $dataForm
     *
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function sendMail($dataForm)
    {
        /** send mail confirm to customer */
        $this->_email->sendEmail(
            $this->_email->getConfirmTemplate(),
            $dataForm['customer_email'],
            $this->getConfirmTemplateParams($dataForm)
        );

        /** send mail to admin */
        $adminEmails = $this->_email->getToAdminEmail();

        if ($adminEmails) {
            if ($dataForm['product_type'] === Grouped::TYPE_CODE || $dataForm['product_type'] === Bundle::TYPE_CODE) {
                $productId = $dataForm['parent_id'];
            } else {
                $productId = $dataForm['product_id'];
            }

            $product = $this->_helperData->getProductById($productId);

            foreach ($adminEmails as $adminEmail) {
                $this->_email->sendEmail(
                    'mppricebargain_email_notify_admin',
                    $adminEmail,
                    ['productName' => $product->getName()]
                );
            }
        }
    }

    /**
     * @param array $dataForm
     *
     * @return array
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function getConfirmTemplateParams($dataForm)
    {
        $data = [];
        if (isset($dataForm['product_type'])
            && ($dataForm['product_type'] === Grouped::TYPE_CODE || $dataForm['product_type'] === Bundle::TYPE_CODE)) {
            foreach ($dataForm['product_id'] as $productId => $item) {
                if ($dataForm['bargain_price'][$productId] && $dataForm['bargain_qty'][$productId]) {
                    $data['items'][] = $this->processConfirmTemplateParams($productId, $dataForm);
                }
            }
        } else {
            $data['items'][] = $this->processConfirmTemplateParams($dataForm['product_id'], $dataForm);
        }

        $data['customerName'] = $dataForm['customer_name'];

        return $data;
    }

    /**
     * @param string|int $productId
     * @param array $dataForm
     *
     * @return array
     * @throws NoSuchEntityException
     * @throws LocalizedException
     */
    public function processConfirmTemplateParams($productId, $dataForm)
    {
        /** @var Product $product */
        $product   = $this->_helperData->getProductById($productId);
        $image_url = $this->_imageHelper->init($product, 'product_thumbnail_image')->getUrl();

        if (isset($dataForm['product_type'])
            && ($dataForm['product_type'] === Grouped::TYPE_CODE || $dataForm['product_type'] === Bundle::TYPE_CODE)) {
            $bargainPrice = $dataForm['bargain_price'][$productId];
            $bargainQty   = $dataForm['bargain_qty'][$productId];
        } else {
            $bargainPrice = $dataForm['bargain_price'];
            $bargainQty   = $dataForm['bargain_qty'];
        }

        return [
            'productName'  => $product->getName(),
            'thumbnail'    => $image_url,
            'customerName' => isset($dataForm['customer_name']) ? $dataForm['customer_name'] : '',
            'bargainPrice' => $this->_helperData->getCurrency() . $bargainPrice,
            'bargainQty'   => $this->_helperData->isBargainQty() ? $bargainQty : null
        ];
    }
}
