<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_NameYourPrice
 * @copyright   Copyright (c) Mageplaza (https://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\NameYourPrice\Plugin\Controller\Cart;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Checkout\Controller\Cart\Add;
use Magento\Checkout\Model\Cart as CustomerCart;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Data\Form\FormKey\Validator;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\StoreManagerInterface;
use Mageplaza\NameYourPrice\Block\Product\View\Bargain;
use Mageplaza\NameYourPrice\Helper\Data as HelperData;
use Mageplaza\NameYourPrice\Model\ResourceModel\Requests;
use Magento\Framework\App\ResourceConnection;

/**
 * Class AddToCart
 * @package Mageplaza\NameYourPrice\Plugin\Controller\Cart
 */
class AddToCart extends Add
{
    /**
     * @var Requests
     */
    protected $_requestResource;

    /**
     * @var HelperData
     */
    protected $_helperData;

    /**
     * @var Bargain
     */
    protected $_bargain;

    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;

    /**
     * AddToCart constructor.
     *
     * @param Context $context
     * @param ScopeConfigInterface $scopeConfig
     * @param Session $checkoutSession
     * @param StoreManagerInterface $storeManager
     * @param Validator $formKeyValidator
     * @param CustomerCart $cart
     * @param ProductRepositoryInterface $productRepository
     * @param HelperData $helperData
     * @param Requests $requestResource
     * @param Bargain $bargain
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        Context $context,
        ScopeConfigInterface $scopeConfig,
        Session $checkoutSession,
        StoreManagerInterface $storeManager,
        Validator $formKeyValidator,
        CustomerCart $cart,
        ProductRepositoryInterface $productRepository,
        HelperData $helperData,
        Requests $requestResource,
        Bargain $bargain,
        ResourceConnection $resourceConnection
    ) {
        $this->_helperData        = $helperData;
        $this->_requestResource   = $requestResource;
        $this->_bargain           = $bargain;
        $this->resourceConnection = $resourceConnection;

        parent::__construct(
            $context,
            $scopeConfig,
            $checkoutSession,
            $storeManager,
            $formKeyValidator,
            $cart,
            $productRepository
        );
    }

    /**
     * @param Add $subject
     * @param callable $proceed
     *
     * @return ResponseInterface|ResultInterface
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function aroundExecute(Add $subject, callable $proceed)
    {
        if (!$this->_helperData->isEnabled() || !$this->_helperData->isBargainQty()) {
            return $proceed();
        }

        $childId   = $this->_request->getParam('mpb-child-product');
        $productId = $childId ?: $this->_request->getParam('product');
        $qty       = $this->_request->getParam('qty');
        $requestId = $this->_request->getParam('mpb-request-id');
        $options   = $this->getRequest()->getParam('options');

        if ($this->getRequest()->getParam('super_group')) {
            if ($this->canAddToCartGroup()) {
                return $proceed();
            }

            return $this->goBack();
        }

        if ($this->getRequest()->getParam('bundle_option')) {
            if ($this->canAddToCartBundle()) {
                return $proceed();
            }

            return $this->goBack();
        }

        $collection = $this->_bargain->getBargainCollection(
            [HelperData::STATUS_APPROVED],
            $productId
        );

        if ($collection && $collection->getSize() > 0) {
            $requestId = $collection->getFirstItem()['request_id'];
        }

        $collection = $this->_bargain->getBargainCollection(
            [HelperData::STATUS_APPROVED],
            $productId,
            $requestId
        );

        if ($collection && $collection->getSize() > 0) {
            $qtyBargain     = (int) $collection->getFirstItem()['bargain_qty'];
            $optionsBargain = $collection->getFirstItem()['options_bundle'];

            if ($qtyBargain === 1 && $qty === null) {
                return $proceed();
            }

            if ($qty < $qtyBargain) {
                $this->messageManager->addErrorMessage(
                    __('The quantity must be equal to or greater than %1 items.', $qtyBargain)
                );

                return $this->goBack();
            }

            if ($optionsBargain && $optionsBargain !== HelperData::jsonEncode($options)) {
                $this->messageManager->addErrorMessage(
                    __('The custom options should be the same as your bargain request.')
                );

                return $this->goBack();
            }
        }

        return $proceed();
    }

    /**
     * @return bool
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function canAddToCartGroup()
    {
        $childIds     = $this->_request->getParam('mpb-child-product');
        $requestIds   = $this->_request->getParam('mpb-request-id');
        $optionToCart = $this->_request->getParam('super_group');

        foreach ($optionToCart as $productId => $qty) {
            if ($qty && isset($childIds[$productId])) {
                $this->processAddToCart(
                    $productId,
                    $requestIds[$productId],
                    $qty
                );
            }

            continue;
        }

        return true;
    }

    /**
     * @return bool
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function canAddToCartBundle()
    {
        $parentId        = $this->_request->getParam('product');
        $bundleOption    = $this->_request->getParam('bundle_option');
        $bundleOptionQty = $this->_request->getParam('bundle_option_qty');
        $childIds        = $this->_request->getParam('mpb-child-product');
        $requestIds      = $this->_request->getParam('mpb-request-id');
        $selectionTable  = $this->resourceConnection->getTableName('catalog_product_bundle_selection');
        $connection      = $this->resourceConnection->getConnection();
        $select          = $connection->select()->from($selectionTable)
            ->where('selection_id IN (?)', $bundleOption)
            ->where('selection_can_change_qty = 1');
        $products        = $connection->fetchAll($select);

        foreach ($products as $product) {
            if (isset($bundleOptionQty[$product['option_id']], $childIds[$product['product_id']])) {
                if ($this->processAddToCart(
                    $product['product_id'],
                    $requestIds[$product['product_id']],
                    $bundleOptionQty[$product['option_id']],
                    $parentId
                )) {
                    continue;
                }

                return false;
            }

            continue;
        }

        return true;
    }

    /**
     * @param int $productId
     * @param null $requestId
     * @param int $qty
     * @param null $parentId
     *
     * @return false
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function processAddToCart($productId, $requestId, $qty, $parentId = null)
    {
        $collection = $this->_bargain->getBargainCollection(
            [HelperData::STATUS_APPROVED],
            $productId,
            $requestId,
            $parentId
        );

        if ($collection && $collection->getSize() > 0) {
            $qtyBargain  = (int) $collection->getFirstItem()['bargain_qty'];
            $productName = $collection->getFirstItem()['product_name'];

            if ($qty < $qtyBargain) {
                $this->messageManager->addErrorMessage(
                    __(
                        'The quantity of %1 must be equal to or greater than %2 items.',
                        $productName,
                        $qtyBargain
                    )
                );

                return false;
            }
        }

        return true;
    }
}
