<?php
/**
 * Copyright © MageWorx. All rights reserved.
 * See LICENSE.txt for license details.
 */
declare(strict_types=1);

namespace MageWorx\OptionFeatures\Plugin;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\FileSystemException;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Registry;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Checkout\Model\Session;
use Magento\Backend\Model\Session\Quote as BackendQuoteSession;
use Magento\Framework\Validation\ValidationException;
use Magento\InventorySales\Model\PlaceReservationsForSalesEvent;
use Magento\InventorySales\Model\SalesEvent;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;
use Magento\InventorySalesApi\Api\Data\SalesEventInterface;
use Magento\Quote\Api\Data\CartInterface;
use MageWorx\OptionBase\Helper\System as SystemHelper;
use MageWorx\OptionBase\Helper\Data as BaseHelper;
use MageWorx\OptionFeatures\Model\QtyMultiplier;
use Magento\Sales\Api\Data\ShipmentInterface;
use MageWorx\OptionFeatures\Plugin\Shipment\RegisterCurrentShipmentAfterSavePlugin;

class AddQtyMultiplierToReservationInventory
{
    /**
     * @var SerializerInterface
     */
    private $serializer;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;

    /**
     * @var BackendQuoteSession
     */
    protected $backendQuoteSession;

    /**
     * Object Manager instance
     *
     * @var ObjectManagerInterface
     */
    protected $objectManager = null;

    /**
     * @var SystemHelper
     */
    protected $systemHelper;

    /**
     * @var Registry
     */
    protected $registry;

    /**
     * @var BaseHelper
     */
    protected $baseHelper;

    /**
     * @var QtyMultiplier
     */
    protected $qtyMultiplier;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @param SerializerInterface $serializer
     * @param Session $checkoutSession
     * @param BackendQuoteSession $backendQuoteSession
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param ObjectManagerInterface $objectManager
     * @param SystemHelper $systemHelper
     * @param Registry $registry
     * @param BaseHelper $baseHelper
     * @param QtyMultiplier $qtyMultiplier
     * @param \Psr\Log\LoggerInterface $logger
     * @param DataPersistorInterface $dataPersistor
     */
    public function __construct(
        SerializerInterface $serializer,
        Session $checkoutSession,
        BackendQuoteSession $backendQuoteSession,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        ObjectManagerInterface $objectManager,
        SystemHelper $systemHelper,
        Registry $registry,
        BaseHelper $baseHelper,
        QtyMultiplier $qtyMultiplier,
        \Psr\Log\LoggerInterface $logger,
        DataPersistorInterface $dataPersistor,
        ProductRepositoryInterface $productRepository
    ) {
        $this->serializer          = $serializer;
        $this->checkoutSession     = $checkoutSession;
        $this->backendQuoteSession = $backendQuoteSession;
        $this->orderRepository     = $orderRepository;
        $this->objectManager       = $objectManager;
        $this->systemHelper        = $systemHelper;
        $this->registry            = $registry;
        $this->baseHelper          = $baseHelper;
        $this->qtyMultiplier       = $qtyMultiplier;
        $this->logger              = $logger;
        $this->dataPersistor       = $dataPersistor;
        $this->productRepository   = $productRepository;
    }

    /**
     * @param PlaceReservationsForSalesEvent $subject
     * @param \Closure $proceed
     * @param array $items
     * @param SalesChannelInterface $salesChannel
     * @param SalesEvent $salesEvent
     * @return mixed|void
     * @throws LocalizedException
     * @throws CouldNotSaveException
     * @throws FileSystemException
     * @throws InputException
     * @throws NoSuchEntityException
     * @throws ValidationException
     */
    public function aroundExecute(
        PlaceReservationsForSalesEvent $subject,
        \Closure $proceed,
        array $items,
        SalesChannelInterface $salesChannel,
        SalesEvent $salesEvent
    ): void {
        if (empty($items)) {
            $proceed($items, $salesChannel, $salesEvent);

            return;
        }

        if ($this->isOrderPlaceFailedSalesEvent($salesEvent)) {
            $proceed($items, $salesChannel, $salesEvent);

            return;
        }

        $currentItems = $this->getCurrentItemsFromSource($salesEvent);

        if (empty($currentItems)) {
            $proceed($items, $salesChannel, $salesEvent);

            return;
        }

        $reservationBuilder                          = $this->objectManager->get(
            \Magento\InventoryReservationsApi\Model\ReservationBuilderInterface::class
        );
        $appendReservations                          = $this->objectManager->get(
            \Magento\InventoryReservationsApi\Model\AppendReservationsInterface::class
        );
        $getStockBySalesChannel                      = $this->objectManager->get(
            \Magento\InventorySalesApi\Api\GetStockBySalesChannelInterface::class
        );
        $getProductTypesBySkus                       = $this->objectManager->get(
            \Magento\InventoryCatalogApi\Model\GetProductTypesBySkusInterface::class
        );
        $isSourceItemManagementAllowedForProductType = $this->objectManager->get(
            \Magento\InventoryConfigurationApi\Model\IsSourceItemManagementAllowedForProductTypeInterface::class
        );
        $salesEventToArrayConverter                  = $this->objectManager->get(
            \Magento\InventorySales\Model\SalesEventToArrayConverter::class
        );

        $qtyMultiplierData = $this->getMultiplierDataToApply($currentItems, $this->isShipmentSalesEvent($salesEvent));

        if (!$qtyMultiplierData) {
            $proceed($items, $salesChannel, $salesEvent);

            return;
        }

        $productTypes = $getProductTypesBySkus->execute($this->collectAllItemsSku($items));
        $stockId      = $getStockBySalesChannel->execute($salesChannel)->getStockId();
        $multiplier   = $this->isOrderSalesEvent($salesEvent) ? -1 : 1;
        $reservations = [];

        /** @var /Magento\InventorySalesApi\Api\Data\ItemToSellInterface $item */
        foreach ($items as $item) {
            $currentSku           = $item->getSku();
            $skuNotExistInCatalog = !isset($productTypes[$currentSku]);
            if ($skuNotExistInCatalog ||
                $isSourceItemManagementAllowedForProductType->execute($productTypes[$currentSku])) {
                if (isset($qtyMultiplierData[$currentSku])) {
                    $qtyToReservation = $qtyMultiplierData[$currentSku]['qty_multiplier_qty'] * $multiplier;
                } else {
                    $qtyToReservation = (float)$item->getQuantity();
                }

                $reservations[] = $reservationBuilder
                    ->setSku($item->getSku())
                    ->setQuantity($qtyToReservation)
                    ->setStockId($stockId)
                    ->setMetadata($this->serializer->serialize($salesEventToArrayConverter->execute($salesEvent)))
                    ->build();
            }
        }
        $appendReservations->execute($reservations);
    }

    /**
     * @param SalesEventInterface $salesEvent
     * @return array
     * @throws LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    protected function getCurrentItemsFromSource(
        SalesEventInterface $salesEvent
    ): array {
        if ($this->isOrderSalesEvent($salesEvent)) {
            $currentItems = $this->getItemsFromSession();
        } else {
            if ($this->isShipmentSalesEvent($salesEvent)) {
                try {
                    $shipment     = $this->getCurrentShipment();
                    $currentItems = $shipment->getAllItems();
                } catch (LocalizedException $exception) {
                    $this->logger->error($exception->getLogMessage());

                    return [];
                }
            } else {
                $currentItems = $this->orderRepository->get($salesEvent->getObjectId())->getItems();
            }

            foreach ($currentItems as $item) {
                $product = $this->productRepository->getById($item->getProductId());
                $item->setSku($product->getSku());
            };
        }

        return $currentItems;
    }

    /**
     * @param array $currentItems
     * @param bool $isShipmentEvent
     * @return array
     * @throws \Magento\Framework\Exception\FileSystemException
     */
    protected function getMultiplierDataToApply(
        array $currentItems,
        bool $isShipmentEvent
    ): array {
        $qtyMultiplierData = [];

        foreach ($currentItems as $currentItem) {
            if ($isShipmentEvent) {
                $orderItem               = $currentItem->getOrderItem();
                $currentQtyMultiplierQty = $this->qtyMultiplier->getQtyMultiplierQtyForCurrentItemQty(
                    $orderItem,
                    $currentItem->getQty()
                );
                $originalProductSku      = $currentItem->getSku();
            } else {
                $currentQtyMultiplierQty = $currentItem->getBuyRequest()->getQtyMultiplierQty();
                $originalProductSku      = $currentItem->getProduct()->getData('sku');
            }

            if (is_null($currentQtyMultiplierQty) || empty($currentQtyMultiplierQty)) {
                return $qtyMultiplierData;
            }

            if (isset($qtyMultiplierData[$originalProductSku]['qty_multiplier_qty'])) {
                $currentQtyMultiplierQty += $qtyMultiplierData[$originalProductSku]['qty_multiplier_qty'];
            }
            $qtyMultiplierData[$originalProductSku] = ['qty_multiplier_qty' => $currentQtyMultiplierQty];
        }

        return $qtyMultiplierData;
    }

    /**
     * @return ShipmentInterface
     */
    protected function getCurrentShipment(): ShipmentInterface
    {
        $shipment = $this->registry->registry('current_shipment');

        return $shipment ?: $this->dataPersistor->get(RegisterCurrentShipmentAfterSavePlugin::SHIPMENT_KEY);
    }

    /**
     * @param SalesEventInterface $salesEvent
     * @return bool
     */
    private function isShipmentSalesEvent(SalesEventInterface $salesEvent): bool
    {
        return $salesEvent->getType() === SalesEventInterface::EVENT_SHIPMENT_CREATED;
    }

    /**
     * @param SalesEventInterface $salesEvent
     * @return bool
     */
    private function isOrderSalesEvent(SalesEventInterface $salesEvent): bool
    {
        return $salesEvent->getType() === SalesEventInterface::EVENT_ORDER_PLACED;
    }

    /**
     * @param SalesEventInterface $salesEvent
     * @return bool
     */
    private function isOrderPlaceFailedSalesEvent(SalesEventInterface $salesEvent): bool
    {
        // not using constant because magento 2.3.x don't have EVENT_ORDER_PLACE_FAILED as consant
        return $salesEvent->getType() === 'order_place_failed';
    }

    /**
     * @return array
     * @throws LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    private function getItemsFromSession(): array
    {
        $quote = $this->systemHelper->isAdmin()
            ? $this->backendQuoteSession->getQuote()
            : $this->checkoutSession->getQuote();

        if (!$quote instanceof CartInterface) {
            throw new LocalizedException(__('Quote is not set.'));
        }

        return $quote->getAllVisibleItems();
    }

    /**
     * @param array $items
     * @return array
     */
    private function collectAllItemsSku(array $items): array
    {
        $skus = [];
        /** @var \Magento\InventorySalesApi\Api\Data\ItemToSellInterface $item */
        foreach ($items as $item) {
            $skus[] = $item->getSku();
        }

        return $skus;
    }
}
