<?php
/**
 * Copyright © MageWorx. All rights reserved.
 * See LICENSE.txt for license details.
 */

namespace MageWorx\OptionInventory\Model;

use MageWorx\OptionInventory\Model\ResourceModel\Product\Option\Value\CollectionFactory as OptionValueCollectionFactory;

/**
 * Validator model
 *
 * @package MageWorx\OptionInventory\Model
 */
class Validator extends \Magento\Framework\Model\AbstractModel
{
    /**
     * @var ObjectManager
     */
    protected $objectManager;

    /**
     * @var \MageWorx\OptionInventory\Helper\Stock
     */
    protected $stockHelper;

    /**
     * @var OptionValueCollectionFactory
     */
    protected $optionValueCollectionFactory;

    /**
     * Validator constructor.
     *
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \MageWorx\OptionInventory\Helper\Stock $stockHelper
     * @param OptionValueCollectionFactory $optionValueCollectionFactory
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\ObjectManagerInterface $objectManager,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \MageWorx\OptionInventory\Helper\Stock $stockHelper,
        OptionValueCollectionFactory $optionValueCollectionFactory,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->objectManager                = $objectManager;
        $this->stockHelper                  = $stockHelper;
        $this->optionValueCollectionFactory = $optionValueCollectionFactory;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * Validate Requested with Original data
     *
     * @param array $requestedData Requested Option Values
     * @param array $originData Original Option Values
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function validate(array $requestedData, array $originData): void
    {
        foreach ($requestedData as $requestedValue) {
            $originValue = isset($originData[$requestedValue->getId()]) ? $originData[$requestedValue->getId()] : null;
            if (!$this->isAllow($requestedValue, $originValue)) {
                $this->addError($originValue, $requestedValue);
            }
        }
    }

    /**
     * Check if allow original qty add requested qty
     *
     * @param \Magento\Framework\DataObject $requestedValue
     * @param \MageWorx\OptionBase\Api\Data\ProductCustomOptionValuesInterface $originValue
     * @return bool
     */
    protected function isAllow(
        \Magento\Framework\DataObject $requestedValue,
        \MageWorx\OptionBase\Api\Data\ProductCustomOptionValuesInterface $originValue
    ): bool {
        if (!$originValue) {
            return true;
        }

        if (!$originValue->getManageStock()) {
            return true;
        }

        if ($originValue->getQty() <= 0) {
            return false;
        }

        if ($requestedValue->getQty() > $originValue->getQty()) {
            return false;
        }

        return true;
    }

    /**
     * Throw exception
     *
     * @param \MageWorx\OptionBase\Api\Data\ProductCustomOptionValuesInterface $value
     * @param \Magento\Framework\DataObject $requestedValue
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    protected function addError(
        \MageWorx\OptionBase\Api\Data\ProductCustomOptionValuesInterface $value,
        \Magento\Framework\DataObject $requestedValue
    ): void {
        $this->correctData($value);

        if ($value->getProductId()) {
            $formattedQty = $this->stockHelper->isfloatingQty((int)$value->getProductId())
                ? (float)$value->getQty()
                : (int)$value->getQty();
        } else {
            $formattedQty = $value->getQty();
        }
        $e = new \Magento\Framework\Exception\LocalizedException(
            __(
                'We don\'t have as many  "%1" : "%2" - "%3"  as you requested (available qty: "%4").',
                $requestedValue->getName(),
                $requestedValue->getOptionTitle(),
                $requestedValue->getValueTitle(),
                $formattedQty
            )
        );
        throw $e;
    }

    /**
     * Correct some option value fields.
     * For example: 'title' - can be origin or use product name linked by sku.
     *
     * SkuIsValid - this property set the OptionLink module.
     *
     * @param \MageWorx\OptionBase\Api\Data\ProductCustomOptionValuesInterface $value
     */
    protected function correctData(\MageWorx\OptionBase\Api\Data\ProductCustomOptionValuesInterface $value): void
    {
        if ($value->getSkuIsValid()) {
            /** @var \MageWorx\OptionInventory\Model\ResourceModel\Product\Option\Value\Collection $valuesCollection */
            $valuesCollection = $this->optionValueCollectionFactory->create();

            $valuesCollection
                ->addTitleToResult(1)
                ->getValuesByOption($value->getId());

            $item = $valuesCollection->getFirstItem();
            $value->setValueTitle($item->getTitle());
        }
    }

    /**
     * This function checks from where to take away quantity.
     *
     * @param \MageWorx\OptionBase\Api\Data\ProductCustomOptionValuesInterface $value
     * @return string
     */
    public function getItemType(\MageWorx\OptionBase\Api\Data\ProductCustomOptionValuesInterface $value): string
    {
        $optionType  = 'option';
        $productType = 'product';

        if (!isset($value['sku_is_valid'])) {
            return $optionType;
        }

        $skuIsValid = $value['sku_is_valid'];

        if ($skuIsValid) {
            return $productType;
        }

        return $optionType;
    }
}
