<?php
/**
 * Copyright © Magefan (support@magefan.com). All rights reserved.
 * Please visit Magefan.com for license details (https://magefan.com/end-user-license-agreement).
 */

namespace Magefan\GuestToCustomer\Model\Order;

use \Magento\Sales\Api\OrderRepositoryInterface;
use \Magento\Sales\Model\Order\CustomerManagement as OrderCustomerManagement;
use \Magento\Customer\Api\CustomerRepositoryInterface;
use \Magento\Customer\Api\Data\AddressInterfaceFactory;
use \Magento\Customer\Api\Data\RegionInterfaceFactory;
use \Magento\Customer\Api\AccountManagementInterface ;
use \Magento\Framework\DataObject\Copy;
use \Magefan\GuestToCustomer\Model\Config;
use \Magento\Customer\Model\AddressRegistry;

/**
 * Class CustomerManagement. Manage customer info.
 */
class CustomerManagement
{
    /**
     * @var Copy
     */
    protected $objectCopyService;

    /**
     * @var OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var OrderCustomerManagement
     */
    protected $orderCustomerManagement;

    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;

    /**
     * @var AddressInterfaceFactory
     */
    protected $customerAddressFactory;

    /**
     * @var RegionInterfaceFactory
     */
    protected $regionFactory;

    /**
     * @var AccountManagementInterface
     */
    protected $accountManagement;

    /**
     * @var \Magento\Downloadable\Model\ResourceModel\Link\Purchased\CollectionFactory
     */
    protected $linkPurchasedCollectionFactory;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var AddressRegistry|mixed
     */
    protected $addressRegistry;

    /**
     * CustomerManagement constructor.
     * @param Copy $objectCopyService
     * @param OrderRepositoryInterface $orderRepository
     * @param OrderCustomerManagement $orderCustomerManagement
     * @param CustomerRepositoryInterface $customerRepository
     * @param AddressInterfaceFactory $customerAddressFactory
     * @param RegionInterfaceFactory $regionFactory
     * @param AccountManagementInterface $accountManagement
     * @param array $data
     * @param null $linkPurchasedCollectionFactory
     * @param Config|null $config
     * @param AddressRegistry|null $addressRegistry
     */
    public function __construct(
        Copy $objectCopyService,
        OrderRepositoryInterface $orderRepository,
        OrderCustomerManagement $orderCustomerManagement,
        CustomerRepositoryInterface $customerRepository,
        AddressInterfaceFactory $customerAddressFactory,
        RegionInterfaceFactory $regionFactory,
        AccountManagementInterface $accountManagement,
        array $data = [],
        $linkPurchasedCollectionFactory = null,
        Config $config = null,
        AddressRegistry $addressRegistry = null
    ) {
        $this->objectCopyService = $objectCopyService;
        $this->orderRepository = $orderRepository;
        $this->customerRepository = $customerRepository;
        $this->orderCustomerManagement = $orderCustomerManagement;
        $this->customerAddressFactory = $customerAddressFactory;
        $this->regionFactory = $regionFactory;
        $this->accountManagement = $accountManagement;

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $this->linkPurchasedCollectionFactory = $linkPurchasedCollectionFactory ?: $objectManager->get(
            \Magento\Downloadable\Model\ResourceModel\Link\Purchased\CollectionFactory::class
        );
        $this->config = $config ?: $objectManager->get(
            \Magefan\GuestToCustomer\Model\Config::class
        );

        $this->addressRegistry = $addressRegistry ?: $objectManager->get(
            \Magento\Customer\Model\AddressRegistry::class
        );
    }

    /**
     * Deprecated
     * @param $orderId
     * @return int|null
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Exception\State\InputMismatchException
     */
    public function getCustomerId($orderId)
    {
        return $this->convertGuestOrderById($orderId);
    }

    /**
     * @param $orderId
     * @return int|null
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Exception\State\InputMismatchException
     */
    public function convertGuestOrderById($orderId)
    {
        $order = $this->orderRepository->get($orderId);
        return $this->convertGuestOrder($order);
    }

    /**
     * @param Magento\Sales\Api\Data\OrderInterface $order
     * @return int|null
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Exception\State\InputMismatchException
     */
    public function convertGuestOrder(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $customerId = $order->getCustomerId();
        if (!$customerId) {
            /* Order was placed by guest */

            try {

                $customerEmail = null;
                if ($order->getBillingAddress()) {
                    $customerEmail = $order->getBillingAddress()->getEmail();
                }

                if (!$customerEmail) {
                    $customerEmail = $order->getCustomerEmail();
                }
                $customer = $this->customerRepository->get(
                    $customerEmail,
                    $order->getStore()->getWebsiteId()
                );
            } catch (\Exception $e) {
                // customer not not exist
                $customer = false;
            }

            /* custom fix for empty firstname or lastname */
            if (!$customer || $this->config->isAssignOrderAddressToCustomer()) {
                foreach ($order->getAddresses() as $address) {
                    if (!$address->getFirstname() && $address->getLastname()) {
                        $name = explode(' ', $address->getLastname());
                        $address->setFirstname(
                            count($name) > 1 ? $name[1] : $name[0]
                        );
                    }

                    if (!$address->getLastname() && $address->getFirstname()) {
                        $name = explode(' ', $address->getFirstname());
                        $address->setLastname(
                            count($name) > 1 ? $name[1] : $name[0]
                        );
                    }

                    if (!$address->getLastname()) {
                        $address->setLastname('Customer');
                    }
                    if (!$address->getFirstname()) {
                        $address->setFirstname('Customer');
                    }
                }
            }
            /* end custom fix */

            if (!$customer) {
                /* Fix in case BillingAddress email is empty */
                if (!$order->getBillingAddress()->getEmail()
                    && $order->getCustomerEmail()
                ) {
                    $order->getBillingAddress()->setEmail($order->getCustomerEmail());
                }

                /* If customer with email not exist */
                /* Create new customer  and add address */
                $customer = $this->orderCustomerManagement->create($order->getId());
                $customer = $this->customerRepository->getById($customer->getId());

                /* Use this logic for M2.0-2.1 as they do not have OrderCustomerExtractor */
                $store = $order->getStore();
                if ($store && $store->getWebsiteId()) {
                    $customer->setWebsiteId($store->getWebsiteId());
                }
                $customer->setStoreId($order->getStoreId());
                /* End logic for M2.0-2.1 */

                if ($customerGroupId = $this->config->getCustomerGroupId()) {
                    $customer->setGroupId($customerGroupId);
                }
            }

            if ($this->config->isAssignOrderAddressToCustomer()) {

                /* Add address to the customer account */
                $customerAddresses = (array)$customer->getAddresses();
                foreach ($customerAddresses as $address) {
                    $addressModel = $this->addressRegistry->retrieve($address->getId());
                    $addressModel->setShouldIgnoreValidation(true);
                }

                $orderAddresses = [];
                $shippingAddress = $order->getShippingAddress();
                $billingAddress = $order->getBillingAddress();

                if ($shippingAddress) {
                    $orderAddresses['shipping'] = $shippingAddress;
                    if (!$this->isAddressesEqual($shippingAddress, $billingAddress)) {
                        $orderAddresses['billing'] = $billingAddress;
                    }
                } else {
                    $orderAddresses['billing'] = $billingAddress;
                }

                /* Fix One or more input exceptions have occurred. */
                if ($shippingAddress) {
                    $addressFields = ['street', 'region', 'city', 'postcode'];
                    foreach ($addressFields as $addressField) {
                        if (!$billingAddress->getData($addressField)) {
                            $getMethod = 'get' . ucfirst($addressField);
                            $setMethod = 'set' . ucfirst($addressField);
                            $sValue = $shippingAddress->$getMethod();
                            if ($sValue) {
                                $billingAddress->$setMethod($sValue);
                            }
                        }
                    }
                }
                /* End fix */

                foreach ($orderAddresses as $addressType => $orderAddress) {
                    foreach ($customerAddresses as $customerAddress) {
                        if ($this->isAddressesEqual($orderAddress, $customerAddress)) {
                            //skip duplicates
                            continue 2;
                        }
                    }
                    /* Create new address */
                    $newAddressData = $this->objectCopyService->copyFieldsetToTarget(
                        'order_address',
                        'to_customer_address',
                        $orderAddress,
                        []
                    );
                    $newAddressData = $this->validateAddressCustomAttributes($newAddressData);
                    $newCustomerAddress = $this->customerAddressFactory->create(['data' => $newAddressData]);
                    if (is_string($orderAddress->getRegion())) {
                        /** @var \Magento\Customer\Api\Data\RegionInterface $region */
                        $region = $this->regionFactory->create();
                        $region->setRegion($orderAddress->getRegion());
                        $region->setRegionCode($orderAddress->getRegionCode());
                        $region->setRegionId($orderAddress->getRegionId());
                        $newCustomerAddress->setRegion($region);
                    }

                    if ('shipping' == $addressType || 1 == count($orderAddresses)) {
                        if (!$customer->getDefaultShipping()) {
                            $newCustomerAddress->setIsDefaultShipping(true);
                        }
                    }
                    if ('billing' == $addressType || 1 == count($orderAddresses)) {
                        if (!$customer->getDefaultBilling()) {
                            $newCustomerAddress->setIsDefaultBilling(true);
                        }
                    }

                    /* Add new address */
                    $customerAddresses[] = $newCustomerAddress;
                }

                /* Save customer address */
                $customer->setAddresses($customerAddresses);
            }

            $this->customerRepository->save($customer);

            /* Update order data */
            $order
                ->setCustomerId($customer->getId())
                ->setCustomerIsGuest(0)
                ->setCustomerFirstname($customer->getFirstname())
                ->setCustomerLastname($customer->getLastname());
            $this->orderRepository->save($order);

            /* Get customer id */
            $customerId = $customer->getId();

            /* Downloadable Link Purchased */
            $this->assignPurchasedLinks($order->getId(), $customer->getId());
        }

        return $customerId;
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @param \Magento\Customer\Api\Data\CustomerInterface $newCustomer
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Exception\State\InputMismatchException
     */
    public function changeOrderCustomer(
        \Magento\Sales\Api\Data\OrderInterface $order,
        \Magento\Customer\Api\Data\CustomerInterface $newCustomer
    ) {
        $order->setCustomerFirstname($newCustomer->getFirstname());
        $order->setCustomerLastname($newCustomer->getLastname());
        $order->setCustomerMiddlename($newCustomer->getMiddlename());
        $order->setCustomerPrefix($newCustomer->getPrefix());
        $order->setCustomerSuffix($newCustomer->getSuffix());
        $order->setCustomerEmail($newCustomer->getEmail());

        if ($oldCustomerId = $order->getCustomerId()) {
            $order->setCustomerId($newCustomer->getId());
            $this->orderRepository->save($order);
            $this->assignPurchasedLinks($order->getId(), $newCustomer->getId(), $oldCustomerId);
        } else {
            $this->convertGuestOrder($order);
        }
    }

    /**
     * @param $orderId
     * @param $customerId
     * @param null $oldCustomerId
     */
    public function assignPurchasedLinks($orderId, $customerId, $oldCustomerId = null)
    {
        $linkPurchasedCollection = $this->linkPurchasedCollectionFactory->create()
            ->addFieldToFilter('order_id', ['eq' => $orderId]);
        if (null === $oldCustomerId) {
            $linkPurchasedCollection->addFieldToFilter('customer_id', ['null' => true]);
        } else {
            $linkPurchasedCollection->addFieldToFilter('customer_id', $oldCustomerId);
        }

        foreach ($linkPurchasedCollection as $link) {
            $link->setData('customer_id', $customerId);
            $link->getResource()->save($link);
        }
    }

    /**
     * Return true if 2 addresses are the same
     * @param $address1
     * @param $address2
     * @return bool
     */
    protected function isAddressesEqual($address1, $address2)
    {
        $methods = ['getRegionId', 'getCountryId', 'getStreet', 'getCompany', 'getTelephone', 'getPostcode',
            'getCity', 'getFirstname', 'getLastname', 'getMiddlename', 'getPrefix', 'getSuffix', 'getVatId'];

        foreach ($methods as $method) {
            $v1 = $address1->{$method}();
            $v2 = $address2->{$method}();
            if ('0' == $v1) {
                $v1 = 0;
            }
            if ('0' == $v2) {
                $v2 = 0;
            }
            if ($v1 != $v2) {
                return false;
            }
        }

        return true;
    }

    /**
     * Validate address custom attributes
     * @param $addressAttributes
     * @return array
     */
    private function validateAddressCustomAttributes($addressAttributes)
    {
        if (isset($addressAttributes['custom_attributes']) && is_array($addressAttributes['custom_attributes'])) {
            $customAttributes = $addressAttributes['custom_attributes'];
            foreach ($customAttributes as $key => $value) {
                if (null === $value) {
                    unset($addressAttributes['custom_attributes'][$key]);
                }
            }
        }
        return $addressAttributes;
    }
}
